'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var execa = require('execa');
var fs = require('fs');
var chalk = require('chalk');
var utils = require('./utils');
var execaOptions = {
  maxBuffer: Infinity
};

module.exports = function setupCommands(config) {
  function runCommandAndWriteToFile(cmd, name) {
    utils.log(chalk.yellow('Executing command:') + ' borg ' + name);
    cmd.stderr.pipe(process.stdout);
    return cmd.then(function (res) {
      var msg = 'Command run:\n' + res.cmd;
      utils.log(chalk.magenta(msg));
    });
  }

  function create() {
    if (!config.paths || config.paths.length === 0) {
      return Promise.reject(new Error('The "paths" option must be specified in the config file, and must contain at least one valid path.'));
    }
    var paths = config.paths.filter(fs.existsSync);
    if (!paths || paths.length === 0) {
      return Promise.reject(Error('No valid paths found in the "paths" option.'));
    }
    if (!config.repository) {
      return Promise.reject(new Error('The "repository" option must be specified in the config file.'));
    }

    var exclude = config.exclude ? config.exclude.reduce(function (pre, cur) {
      return pre.concat(['--exclude', cur]);
    }, []) : [];
    var options = config.create && config.create.options ? config.create.options : [];
    return runCommandAndWriteToFile(execa(config.borgPath || 'borg', ['create', '--stats', '-v', '--list'].concat(_toConsumableArray(options), _toConsumableArray(exclude), [config.repository + '::' + (config.archivePrefix || '') + config.archiveName], _toConsumableArray(paths)), execaOptions), 'create');
  }

  function check() {
    if (!config.check) {
      return Promise.resolve();
    }
    return runCommandAndWriteToFile(execa(config.borgPath || 'borg', ['check', '-v', '--show-rc', '--debug'].concat(_toConsumableArray(config.check.options || []), [config.repository]), execaOptions), 'check');
  }

  function prune() {
    if (!config.prune) {
      return Promise.resolve();
    }

    var prefix = config.archivePrefix ? ['--prefix', config.archivePrefix] : [];
    return runCommandAndWriteToFile(execa(config.borgPath || 'borg', ['prune', config.repository, '-v', '--show-rc', '--list'].concat(prefix, _toConsumableArray(config.prune.options || [])), execaOptions), 'prune');
  }

  return {
    check: check,
    prune: prune,
    create: create
  };
};