'use strict';

var dateFormat = require('dateformat');

function run(config, archiveName) {
  if (!config) {
    return Promise.reject(new Error('A valid configuration object must be passed in.'));
  }

  if (config.env) {
    for (var key in config.env) {
      process.env[key] = config.env[key];
    }
  }
  config.archiveName = archiveName || dateFormat(new Date(), 'yyyy-mm-dd-HHMMss');
  var commands = require('./commands')(config);
  var data = {
    archiveName: config.archiveName
  };
  return commands.create().then(commands.check).then(commands.prune).then(function () {
    return new Promise(function (resolve) {
      if (config.onFinish && typeof config.onFinish === 'function') {
        config.onFinish(null, data, resolve);
        return;
      }
      resolve();
    });
  }).catch(function (err) {
    return new Promise(function (resolve, reject) {
      if (config.onFinish && typeof config.onFinish === 'function') {
        config.onFinish(err, data, reject.bind(null, err));
        return;
      }
      reject(err);
    });
  });
}

module.exports = run;